"""
Works out statistics for rotor data
"""

import sys
import cPickle
import numpy as np
import matplotlib.pyplot as plt

rotor_datafile = 'mrot_l1500'

with open('%s.p' % rotor_datafile, 'rb') as f:
    data = cPickle.load(f)

ECG_start = data['ECG Start']  # (x, y)
ECG_end = data['ECG End']  # (x, y), y loop - ('NA', 'Y LOOP'), X loop - ('X Loop', y)
rotor_position = data['Rotor Position']  # (x, y)
ECG_count = data['ECG Counter']  # Total
ECG_count[ECG_count == 'Limit Hit'] = 30
Y_constraint_check = data['Y Constraint Check']  # either 0, 1, 3
X_constraint_check = data['X Constraint Check']  # either 0, 1, 3
Y_zero_check = data['Y Zero Check']
X_zero_check = data['X Zero Check']
Y_loop_check = data['Y Loop']
X_loop_check = data['X Loop']

ECG_count[0] = ['Limit Hit', 0]

zipped_data = zip(ECG_start, ECG_end, rotor_position, ECG_count, Y_constraint_check, X_constraint_check,
                  Y_zero_check, X_zero_check, Y_loop_check, X_loop_check)

# ECG Counts filtered
c1 = [x[3][0] for x in zipped_data]
c1_fail_count = c1.count('Limit Hit')
c1 = [x for x in c1 if x != 'Limit Hit']

c2 = [x[3][1] for x in zipped_data]
c2_fail_count = c2.count('Limit Hit')
c2 = [x for x in c2 if x != 'Limit Hit']

# ECG statistics
average_ecg_1 = np.mean(c1)
std_ecg_1 = np.std(c1, ddof=1)
average_ecg_2 = np.mean(c2)
std_ecg_2 = np.std(c2, ddof=1)

pure_locators = [x for x in zipped_data if x[4] == [0, 0] and x[5] == [0, 0] and x[6] == [0, 0] and x[7] == [0, 0] and x[8] == [0, 0] and x[9] == [0, 0]]
no_constraint_limit_locators = [x for x in zipped_data if x[4] == [0, 0] and x[5] == [0, 0]]
no_loop_locators = [x for x in zipped_data if x[8] == [0, 0] and x[9] == [0, 0]]
no_zero_locators = [x for x in zipped_data if x[6] == [0, 0] and x[7] == [0, 0]]

# Distance between rotor and final ecg position
def distance(vectors):
    """
    :param rotors: tuple
    :param ecgs: tuple
    :return:
    """
    x_vector1 = vectors[0][0]
    x_vector2 = vectors[1][0]
    y_vector1 = vectors[0][1]
    y_vector2 = vectors[1][1]

    distance_1 = np.sqrt(y_vector1**2 + x_vector1**2)
    distance_2 = np.sqrt(y_vector2**2 + x_vector2**2)
    return distance_1, distance_2


def y_vector_wrapper(y_vector):
    """
    :return:
    """
    if y_vector > 100:
        y_vector -= 200
    if y_vector <= -100:
        y_vector += 200
    return y_vector


def vector_distance(rotor1, ecg1, rotor2, ecg2):
    """
    :param rotor1:
    :param ecg1:
    :param rotor2:
    :param ecg2:
    :return:
    """
    try:
        y_vector1 = y_vector_wrapper(ecg1[1] - rotor1[1])  # Link A
        y_vector2 = y_vector_wrapper(ecg2[1] - rotor1[1])  # Link A
        y_vector3 = y_vector_wrapper(ecg1[1] - rotor2[1])  # Link B
        y_vector4 = y_vector_wrapper(ecg2[1] - rotor2[1])  # Link B

        if np.abs(y_vector1) < np.abs(y_vector2):
            x_vector1 = ecg1[0] - rotor1[0]
            true_y_vector1 = y_vector1
        else:
            x_vector1 = ecg2[0] - rotor1[0]
            true_y_vector1 = y_vector2

        distance_1 = (x_vector1, true_y_vector1)

        if np.abs(y_vector3) < np.abs(y_vector4):
            x_vector2 = ecg1[0] - rotor2[0]
            true_y_vector2 = y_vector3
        else:
            x_vector2 = ecg2[0] - rotor2[0]
            true_y_vector2 = y_vector4

        distance_2 = (x_vector2, true_y_vector2)

    except:

        if not ecg1 and not ecg2:
            distance_1 = 'Fail'
            distance_2 = 'Fail'

        elif not ecg1 and ecg2:

            y_vector2 = y_vector_wrapper(ecg2[1] - rotor1[1])  # Link A
            y_vector4 = y_vector_wrapper(ecg2[1] - rotor2[1])  # Link B
                
            x_vector2 = ecg2[0] - rotor2[0]

            if np.abs(y_vector2) < np.abs(y_vector4):
                true_y_vector2 = y_vector2
            else:
                true_y_vector2 = y_vector4

            distance_1 = 'Fail'
            distance_2 = (x_vector2, true_y_vector2)

        elif not ecg2 and ecg1:

            y_vector1 = y_vector_wrapper(ecg1[1] - rotor1[1])  # Link A
            y_vector3 = y_vector_wrapper(ecg1[1] - rotor2[1])  # Link B

            x_vector1 = ecg1[0] - rotor2[0]

            if np.abs(y_vector1) < np.abs(y_vector3):
                true_y_vector1 = y_vector1
            else:
                true_y_vector1 = y_vector3

            distance_2 = 'Fail'
            distance_1 = (x_vector1, true_y_vector1)

    return distance_1, distance_2


def prob_calculator(distances, count_type, first_fail_count):

    x_limits = np.arange(-4,33)
    y_limits = np.arange(-3,4)
    target_true = []
    target_dist = []
    for i in distances:
        if i != 'Fail':
            x = i[0] in x_limits
            y = i[1] in y_limits
            t = x * y
        else:
            t = False
        target_true.append(t)

    # first rotor
    if count_type == 1:
        target_true = target_true.count(1)/float(len(target_true))

    # second rotor
    if count_type == 2:
        target_true = target_true.count(1)/float((len(target_true) - first_fail_count))

    return target_true

vectors = [vector_distance(x[2][0], x[1][0], x[2][1], x[1][1]) for x in zipped_data]
vector_unp = [x for sublist in vectors for x in sublist]

firstE_vect = [x[0] for x in vectors]
secondE_vect = [x[1] for x in vectors]

first_failcount = firstE_vect.count('Fail')
second_failcount = secondE_vect.count('Fail')

print("Average total number of ECGS to find first rotor: {:.3f} ".format(average_ecg_1) \
      + u"\u00B1" + " {:.3f}".format(std_ecg_1))
print("Average total number of ECGS to find second rotor: {:.3f} ".format(average_ecg_2) \
      + u"\u00B1" + " {:.3f}".format(std_ecg_2))
print("Rotor 1 Prediction Probability: {:.2f}%".format(prob_calculator(firstE_vect, count_type=1, first_fail_count=first_failcount) * 100))
print("Rotor 2 Prediction Probability: {:.2f}%".format(prob_calculator(secondE_vect, count_type=2, first_fail_count=first_failcount) * 100))
